import ssl
import smtplib

from urllib.parse import urlparse

from argos.logging import logger

# XXX Implement mail alerts https://framagit.org/framasoft/framaspace/argos/-/issues/15
# XXX Implement gotify alerts https://framagit.org/framasoft/framaspace/argos/-/issues/16

def handle_alert(config, result, task, severity):
    """Dispatch alert through configured alert channels"""
    msg = f"task={task.id}, status={result.status}, {severity=}"
    logger.error("Alerting stub: %s", msg)

    if 'mail' in config and 'addresses' in config['mail']:
        logger.debug('Notify of failure by mail')

        msg = f"""\
Subject: Argos {urlparse(task.url).netloc} status {severity}

URL:    {task.url}
Check:  {task.check}
Status: {severity}
Time:   {result.submitted_at}
"""
        notify_by_mail(msg=msg, config=config['mail'])


def notify_by_mail(msg: str, config: dict) -> None:
    """Notify by mail

    Keyword argument:
    msg    -- string, the mail to send (including subject)
    config -- dict, configuration of mail system
    """
    if 'mailfrom' not in config:
        logger.error('No "mailfrom" address in mail config. '
                     'Not sending mail.')
        return None
    if 'host' not in config:
        config['host'] = '127.0.0.1'
    if 'port' not in config:
        config['port'] = 25
    if 'ssl' not in config:
        config['ssl'] = False
    if 'starttls' not in config:
        config['starttls'] = False
    if config['ssl']:
        logger.debug('Mail notification: SSL')
        context = ssl.create_default_context()
        smtp = smtplib.SMTP_SSL(host=config['host'],
                                port=config['port'],
                                context=context)
    else:
        smtp = smtplib.SMTP(host=config['host'],  # type: ignore
                            port=config['port'])
        if config['starttls']:
            logger.debug('Mail notification: STARTTLS')
            context = ssl.create_default_context()
            smtp.starttls(context=context)

    if 'auth' in config:
        if 'login' not in config['auth'] \
           or 'password' not in config['auth']:
            logger.warning('Mail credentials are incomplete. '
                           'No mail authentication can be done. '
                           'Not sending mail.')
            return None

        logger.debug('Mail notification: authentification')
        smtp.login(config['auth']['login'],
                   config['auth']['password'])

    for address in config['addresses']:
        logger.debug('Sending mail to %s', address)
        logger.debug(msg)
        smtp.sendmail(config['from'], address, msg)

    return None
